@@ -19,7 +19,6 @@ from urllib.error import URLError
 from urllib.request import urlopen
 from urllib.parse import urlsplit, uses_relative
 from pathlib import Path, PureWindowsPath, PurePath
-from xml.etree import ElementTree
 
 try:
     import lxml.etree as lxml_etree
@@ -28,8 +27,8 @@ except ImportError:
 
 from xmlschema import fetch_namespaces, fetch_resource, normalize_url, \
     fetch_schema, fetch_schema_locations, XMLResource, XMLResourceError, XMLSchema
-from xmlschema.etree import etree_element, py_etree_element, is_etree_element
-from xmlschema.namespaces import XSD_NAMESPACE
+from xmlschema.etree import ElementTree, etree_element, py_etree_element, is_etree_element
+from xmlschema.names import XSD_NAMESPACE
 from xmlschema.resources import is_url, is_local_url, is_remote_url, \
     url_path_is_file, normalize_locations, LazySelector
 from xmlschema.testing import SKIP_REMOTE_TESTS
@@ -290,6 +289,11 @@ class TestResources(unittest.TestCase):
         resource.load()
         self.assertTrue(resource.text.startswith('<?xml'))
 
+        resource = XMLResource(self.vh_xml_file, lazy=False)
+        resource._url = resource._url[:-12] + 'unknown.xml'
+        with self.assertRaises(XMLResourceError):
+            resource.load()
+
     def test_xml_resource_from_element_tree(self):
         vh_etree = ElementTree.parse(self.vh_xml_file)
         vh_root = vh_etree.getroot()
@@ -350,6 +354,13 @@ class TestResources(unittest.TestCase):
         finally:
             xml_file.close()
 
+        with open(self.vh_xml_file) as fp:
+            resource = XMLResource(fp)
+        self.assertIsNone(resource.text)
+
+        with self.assertRaises(XMLResourceError):
+            resource.load()
+
     def test_xml_resource_from_file(self):
         with open(self.vh_xsd_file) as schema_file:
             resource = XMLResource(schema_file, lazy=False)
