@@ -18,14 +18,14 @@ from elementpath import XPath2Parser, XPathContext, ElementPathError, \
     translate_pattern, RegexError
 
 from ..etree import etree_element
-from ..qnames import XSD_LENGTH, XSD_MIN_LENGTH, XSD_MAX_LENGTH, XSD_ENUMERATION, \
+from ..names import XSD_LENGTH, XSD_MIN_LENGTH, XSD_MAX_LENGTH, XSD_ENUMERATION, \
     XSD_INTEGER, XSD_WHITE_SPACE, XSD_PATTERN, XSD_MAX_INCLUSIVE, XSD_MAX_EXCLUSIVE, \
     XSD_MIN_INCLUSIVE, XSD_MIN_EXCLUSIVE, XSD_TOTAL_DIGITS, XSD_FRACTION_DIGITS, \
     XSD_ASSERTION, XSD_DECIMAL, XSD_EXPLICIT_TIMEZONE, XSD_NOTATION_TYPE, \
     XSD_BASE64_BINARY, XSD_HEX_BINARY, XSD_QNAME
-from ..helpers import count_digits
 
 from .exceptions import XMLSchemaValidationError, XMLSchemaDecodeError
+from .helpers import count_digits
 from .xsdbase import XsdComponent
 
 
@@ -44,9 +44,9 @@ class XsdFacet(XsdComponent):
 
     def __call__(self, value):
         try:
-            yield from self.validator(value)
-        except (TypeError, ValueError) as err:
-            yield XMLSchemaValidationError(self, value, str(err))
+            self.validator(value)
+        except TypeError as err:
+            raise XMLSchemaValidationError(self, value, str(err)) from None
 
     def _parse(self):
         super(XsdFacet, self)._parse()
@@ -59,7 +59,7 @@ class XsdFacet(XsdComponent):
             self._parse_value(self.elem)
         except (KeyError, ValueError, XMLSchemaDecodeError) as err:
             self.value = None
-            self.parse_error(str(err))
+            self.parse_error(err)
         else:
             if base_facet is not None and base_facet.fixed and \
                     base_facet.value is not None and self.value != base_facet.value:
@@ -123,13 +123,17 @@ class XsdWhiteSpaceFacet(XsdFacet):
             self.parse_error("attribute 'value' must be one of "
                              "('preserve', 'replace', 'collapse').")
 
-    def replace_white_space_validator(self, x):
-        if '\t' in x or '\n' in x:
-            yield XMLSchemaValidationError(self, x)
+    def replace_white_space_validator(self, value):
+        if '\t' in value or '\n' in value:
+            raise XMLSchemaValidationError(
+                self, value, "value contains tabs or newlines"
+            )
 
-    def collapse_white_space_validator(self, x):
-        if '\t' in x or '\n' in x or '  ' in x:
-            yield XMLSchemaValidationError(self, x)
+    def collapse_white_space_validator(self, value):
+        if '\t' in value or '\n' in value or '  ' in value:
+            raise XMLSchemaValidationError(
+                self, value, "value contains non collapsed white spaces"
+            )
 
 
 class XsdLengthFacet(XsdFacet):
@@ -163,18 +167,24 @@ class XsdLengthFacet(XsdFacet):
         else:
             self.validator = self.length_validator
 
-    def length_validator(self, x):
-        if len(x) != self.value:
-            yield XMLSchemaValidationError(self, x, "length has to be %r." % self.value)
+    def length_validator(self, value):
+        if len(value) != self.value:
+            raise XMLSchemaValidationError(
+                self, value, "length has to be {!r}".format(self.value)
+            )
 
-    def hex_length_validator(self, x):
-        if len(x) != self.value * 2:
-            yield XMLSchemaValidationError(self, x, "binary length has to be %r." % self.value)
+    def hex_length_validator(self, value):
+        if len(value) != self.value * 2:
+            raise XMLSchemaValidationError(
+                self, value, "binary length has to be {!r}".format(self.value)
+            )
 
-    def base64_length_validator(self, x):
-        x = x.replace(' ', '')
-        if (len(x) // 4 * 3 - (x[-1] == '=') - (x[-2] == '=')) != self.value:
-            yield XMLSchemaValidationError(self, x, "binary length has to be %r." % self.value)
+    def base64_length_validator(self, value):
+        value = value.replace(' ', '')
+        if (len(value) // 4 * 3 - (value[-1] == '=') - (value[-2] == '=')) != self.value:
+            raise XMLSchemaValidationError(
+                self, value, "binary length has to be {!r}".format(self.value)
+            )
 
 
 class XsdMinLengthFacet(XsdFacet):
@@ -206,23 +216,24 @@ class XsdMinLengthFacet(XsdFacet):
         elif primitive_type.name not in (XSD_QNAME, XSD_NOTATION_TYPE):
             self.validator = self.min_length_validator
 
-    def min_length_validator(self, x):
-        if len(x) < self.value:
-            yield XMLSchemaValidationError(
-                self, x, "length cannot be lesser than %r." % self.value
+    def min_length_validator(self, value):
+        if len(value) < self.value:
+            raise XMLSchemaValidationError(
+                self, value, "value length cannot be lesser than {!r}".format(self.value)
             )
 
-    def hex_min_length_validator(self, x):
-        if len(x) < self.value * 2:
-            yield XMLSchemaValidationError(
-                self, x, "binary length cannot be lesser than %r." % self.value
+    def hex_min_length_validator(self, value):
+        if len(value) < self.value * 2:
+            raise XMLSchemaValidationError(
+                self, value, "binary length cannot be lesser than {!r}".format(self.value)
             )
 
-    def base64_min_length_validator(self, x):
-        x = x.replace(' ', '')
-        if (len(x) // 4 * 3 - (x[-1] in ('=', 61)) - (x[-2] in ('=', 61))) < self.value:
-            yield XMLSchemaValidationError(
-                self, x, "binary length cannot be lesser than %r." % self.value
+    def base64_min_length_validator(self, value):
+        value = value.replace(' ', '')
+        if (len(value) // 4 * 3 - (value[-1] in ('=', 61)) - (value[-2] in ('=', 61))) \
+                < self.value:
+            raise XMLSchemaValidationError(
+                self, value, "binary length cannot be lesser than {!r}".format(self.value)
             )
 
 
@@ -255,23 +266,23 @@ class XsdMaxLengthFacet(XsdFacet):
         elif primitive_type.name not in (XSD_QNAME, XSD_NOTATION_TYPE):
             self.validator = self.max_length_validator
 
-    def max_length_validator(self, x):
-        if len(x) > self.value:
-            yield XMLSchemaValidationError(
-                self, x, "length cannot be greater than %r." % self.value
+    def max_length_validator(self, value):
+        if len(value) > self.value:
+            raise XMLSchemaValidationError(
+                self, value, "value length cannot be greater than {!r}".format(self.value)
             )
 
-    def hex_max_length_validator(self, x):
-        if len(x) > self.value * 2:
-            yield XMLSchemaValidationError(
-                self, x, "binary length cannot be greater than %r." % self.value
+    def hex_max_length_validator(self, value):
+        if len(value) > self.value * 2:
+            raise XMLSchemaValidationError(
+                self, value, "binary length cannot be greater than {!r}".format(self.value)
             )
 
-    def base64_max_length_validator(self, x):
-        x = x.replace(' ', '')
-        if (len(x) // 4 * 3 - (x[-1] == '=') - (x[-2] == '=')) > self.value:
-            yield XMLSchemaValidationError(
-                self, x, "binary length cannot be greater than %r." % self.value
+    def base64_max_length_validator(self, value):
+        value = value.replace(' ', '')
+        if (len(value) // 4 * 3 - (value[-1] == '=') - (value[-2] == '=')) > self.value:
+            raise XMLSchemaValidationError(
+                self, value, "binary length cannot be greater than {!r}".format(self.value)
             )
 
 
@@ -311,10 +322,10 @@ class XsdMinInclusiveFacet(XsdFacet):
     def __call__(self, value):
         try:
             if value < self.value:
-                reason = "value has to be greater or equal than %r." % self.value
-                yield XMLSchemaValidationError(self, value, reason)
-        except (TypeError, ValueError) as err:
-            yield XMLSchemaValidationError(self, value, str(err))
+                reason = "value has to be greater or equal than {!r}".format(self.value)
+                raise XMLSchemaValidationError(self, value, reason)
+        except TypeError as err:
+            raise XMLSchemaValidationError(self, value, str(err)) from None
 
 
 class XsdMinExclusiveFacet(XsdFacet):
@@ -353,10 +364,10 @@ class XsdMinExclusiveFacet(XsdFacet):
     def __call__(self, value):
         try:
             if value <= self.value:
-                reason = "value has to be greater than %r." % self.value
-                yield XMLSchemaValidationError(self, value, reason)
-        except (TypeError, ValueError) as err:
-            yield XMLSchemaValidationError(self, value, str(err))
+                reason = "value has to be greater than {!r}".format(self.value)
+                raise XMLSchemaValidationError(self, value, reason)
+        except TypeError as err:
+            raise XMLSchemaValidationError(self, value, str(err)) from None
 
 
 class XsdMaxInclusiveFacet(XsdFacet):
@@ -395,10 +406,10 @@ class XsdMaxInclusiveFacet(XsdFacet):
     def __call__(self, value):
         try:
             if value > self.value:
-                reason = "value has to be lesser or equal than %r." % self.value
-                yield XMLSchemaValidationError(self, value, reason)
-        except (TypeError, ValueError) as err:
-            yield XMLSchemaValidationError(self, value, str(err))
+                reason = "value has to be lesser or equal than {!r}".format(self.value)
+                raise XMLSchemaValidationError(self, value, reason)
+        except TypeError as err:
+            raise XMLSchemaValidationError(self, value, str(err)) from None
 
 
 class XsdMaxExclusiveFacet(XsdFacet):
@@ -437,10 +448,10 @@ class XsdMaxExclusiveFacet(XsdFacet):
     def __call__(self, value):
         try:
             if value >= self.value:
-                reason = "value has to be lesser than %r" % self.value
-                yield XMLSchemaValidationError(self, value, reason)
-        except (TypeError, ValueError) as err:
-            yield XMLSchemaValidationError(self, value, str(err))
+                reason = "value has to be lesser than {!r}".format(self.value)
+                raise XMLSchemaValidationError(self, value, reason)
+        except TypeError as err:
+            raise XMLSchemaValidationError(self, value, str(err)) from None
 
 
 class XsdTotalDigitsFacet(XsdFacet):
@@ -463,13 +474,14 @@ class XsdTotalDigitsFacet(XsdFacet):
             raise ValueError("'value' must be greater or equal than 1")
         self.validator = self.total_digits_validator
 
-    def total_digits_validator(self, x):
+    def total_digits_validator(self, value):
         try:
-            if operator.add(*count_digits(x)) > self.value:
-                reason = "the number of digits is greater than %r." % self.value
-                yield XMLSchemaValidationError(self, x, reason)
-        except (TypeError, ValueError, ArithmeticError) as err:
-            yield XMLSchemaValidationError(self, x, str(err))
+            if operator.add(*count_digits(value)) > self.value:
+                reason = "the number of digits has to be lesser or equal " \
+                         "than {!r}".format(self.value)
+                raise XMLSchemaValidationError(self, value, reason)
+        except (TypeError, ArithmeticError) as err:
+            raise XMLSchemaValidationError(self, value, str(err)) from None
 
 
 class XsdFractionDigitsFacet(XsdFacet):
@@ -502,13 +514,14 @@ class XsdFractionDigitsFacet(XsdFacet):
                              "for types derived from xs:integer.")
         self.validator = self.fraction_digits_validator
 
-    def fraction_digits_validator(self, x):
+    def fraction_digits_validator(self, value):
         try:
-            if count_digits(x)[1] > self.value:
-                reason = "the number of fraction digits is greater than %r." % self.value
-                yield XMLSchemaValidationError(self, x, reason)
-        except (TypeError, ValueError, ArithmeticError) as err:
-            yield XMLSchemaValidationError(self, x, str(err))
+            if count_digits(value)[1] > self.value:
+                reason = "the number of fraction digits has to be lesser " \
+                         "or equal than {!r}".format(self.value)
+                raise XMLSchemaValidationError(self, value, reason)
+        except (TypeError, ArithmeticError) as err:
+            raise XMLSchemaValidationError(self, value, str(err)) from None
 
 
 class XsdExplicitTimezoneFacet(XsdFacet):
@@ -536,16 +549,16 @@ class XsdExplicitTimezoneFacet(XsdFacet):
                 "attribute 'value' must be one of ('required', 'prohibited', 'optional')."
             )
 
-    def required_timezone_validator(self, x):
-        if x.tzinfo is None:
-            yield XMLSchemaValidationError(
-                self, x, "time zone required for value %r." % self.value
+    def required_timezone_validator(self, value):
+        if value.tzinfo is None:
+            raise XMLSchemaValidationError(
+                self, value, "time zone required for value {!r}".format(self.value)
             )
 
-    def prohibited_timezone_validator(self, x):
-        if x.tzinfo is not None:
-            yield XMLSchemaValidationError(
-                self, x, "time zone prohibited for value %r." % self.value
+    def prohibited_timezone_validator(self, value):
+        if value.tzinfo is not None:
+            raise XMLSchemaValidationError(
+                self, value, "time zone prohibited for value {!r}".format(self.value)
             )
 
 
@@ -621,7 +634,6 @@ class XsdEnumerationFacets(MutableSequence, XsdFacet):
     def __call__(self, value):
         if value in self.enumeration:
             return
-
         try:
             if math.isnan(value) and any(math.isnan(x) for x in self.enumeration):
                 return
@@ -630,8 +642,8 @@ class XsdEnumerationFacets(MutableSequence, XsdFacet):
         except TypeError:
             pass
 
-        reason = "invalid value %r, it must be one of %r" % (value, self.enumeration)
-        yield XMLSchemaValidationError(self, value, reason=reason)
+        reason = "value must be one of {!r}".format(self.enumeration)
+        raise XMLSchemaValidationError(self, value, reason)
 
 
 class XsdPatternFacets(MutableSequence, XsdFacet):
@@ -701,10 +713,10 @@ class XsdPatternFacets(MutableSequence, XsdFacet):
     def __call__(self, text):
         try:
             if all(pattern.match(text) is None for pattern in self.patterns):
-                msg = "value doesn't match any pattern of %r."
-                yield XMLSchemaValidationError(self, text, reason=msg % self.regexps)
+                reason = "value doesn't match any pattern of {!r}".format(self.regexps)
+                raise XMLSchemaValidationError(self, text, reason)
         except TypeError as err:
-            yield XMLSchemaValidationError(self, text, str(err))
+            raise XMLSchemaValidationError(self, text, str(err)) from None
 
     @property
     def regexps(self):
@@ -754,7 +766,7 @@ class XsdAssertionFacet(XsdFacet):
         try:
             self.path = self.elem.attrib['test']
         except KeyError as err:
-            self.parse_error(str(err), elem=self.elem)
+            self.parse_error(err)
             self.path = 'true()'
 
         try:
@@ -775,17 +787,17 @@ class XsdAssertionFacet(XsdFacet):
         try:
             self.token = self.parser.parse(self.path)
         except ElementPathError as err:
-            self.parse_error(err, elem=self.elem)
+            self.parse_error(err)
             self.token = self.parser.parse('true()')
 
     def __call__(self, value):
         context = XPathContext(self._root, variables={'value': value})
         try:
             if not self.token.evaluate(context):
-                msg = "value is not true with test path %r."
-                yield XMLSchemaValidationError(self, value, reason=msg % self.path)
+                reason = "value is not true with test path {!r}".format(self.path)
+                raise XMLSchemaValidationError(self, value, reason)
         except ElementPathError as err:
-            yield XMLSchemaValidationError(self, value, reason=str(err))
+            raise XMLSchemaValidationError(self, value, reason=str(err)) from None
 
 
 XSD_10_FACETS_BUILDERS = {
